@@ -42,9 +42,9 @@ impl ToCounterValue for i64 {
 /// Conversion trait for valid values for timers
 ///
 /// This trait must be implemented for any types that are used as timer
-/// values (currently `u64` and `Duration`). This trait is internal to how
-/// values are formatted as part of metrics but is exposed publicly for
-/// documentation purposes.
+/// values (currently `u64`, `Duration`, and `Vec`s of those types).
+/// This trait is internal to how values are formatted as part of metrics
+/// but is exposed publicly for documentation purposes.
 ///
 /// Typical use of Cadence shouldn't require interacting with this trait.
 pub trait ToTimerValue {
@@ -57,6 +57,12 @@ impl ToTimerValue for u64 {
     }
 }
 
+impl ToTimerValue for Vec<u64> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        Ok(MetricValue::PackedUnsigned(self))
+    }
+}
+
 impl ToTimerValue for Duration {
     fn try_to_value(self) -> MetricResult<MetricValue> {
         let as_millis = self.as_millis();
@@ -68,6 +74,18 @@ impl ToTimerValue for Duration {
     }
 }
 
+impl ToTimerValue for Vec<Duration> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        if self.iter().any(|x| x.as_millis() > u64::MAX as u128) {
+            Err(MetricError::from((ErrorKind::InvalidInput, "u64 overflow")))
+        } else {
+            Ok(MetricValue::PackedUnsigned(
+                self.iter().map(|x| x.as_millis() as u64).collect(),
+            ))
+        }
+    }
+}
+
 /// Conversion trait for valid values for gauges
 ///
 /// This trait must be implemented for any types that are used as gauge
@@ -112,9 +130,9 @@ impl ToMeterValue for u64 {
 /// Conversion trait for valid values for histograms
 ///
 /// This trait must be implemented for any types that are used as histogram
-/// values (currently `u64`, `f64`, and `Duration`). This trait is internal
-/// to how values are formatted as part of metrics but is exposed publicly
-/// for documentation purposes.
+/// values (currently `u64`, `f64`, `Duration`, and `Vec`s of those types).
+/// This trait is internal to how values are formatted as part of metrics
+/// but is exposed publicly for documentation purposes.
 ///
 /// Typical use of Cadence shouldn't require interacting with this trait.
 pub trait ToHistogramValue {
@@ -144,12 +162,36 @@ impl ToHistogramValue for Duration {
     }
 }
 
+impl ToHistogramValue for Vec<u64> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        Ok(MetricValue::PackedUnsigned(self))
+    }
+}
+
+impl ToHistogramValue for Vec<f64> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        Ok(MetricValue::PackedFloat(self))
+    }
+}
+
+impl ToHistogramValue for Vec<Duration> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        if self.iter().any(|x| x.as_nanos() > u64::MAX as u128) {
+            Err(MetricError::from((ErrorKind::InvalidInput, "u64 overflow")))
+        } else {
+            Ok(MetricValue::PackedUnsigned(
+                self.iter().map(|x| x.as_nanos() as u64).collect(),
+            ))
+        }
+    }
+}
+
 /// Conversion trait for valid values for distributions
 ///
 /// This trait must be implemented for any types that are used as distribution
-/// values (currently `u64` and `f64`). This trait is internal to how values are
-/// formatted as part of metrics but is exposed publicly for documentation
-/// purposes.
+/// values (currently `u64`, `f64`, and `Vec`s of those types). This trait is
+/// internal to how values are formatted as part of metrics but is exposed
+/// publicly for documentation purposes.
 ///
 /// Typical use of Cadence shouldn't require interacting with this trait.
 pub trait ToDistributionValue {
@@ -168,6 +210,18 @@ impl ToDistributionValue for f64 {
     }
 }
 
+impl ToDistributionValue for Vec<u64> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        Ok(MetricValue::PackedUnsigned(self))
+    }
+}
+
+impl ToDistributionValue for Vec<f64> {
+    fn try_to_value(self) -> MetricResult<MetricValue> {
+        Ok(MetricValue::PackedFloat(self))
+    }
+}
+
 /// Conversion trait for valid values for sets
 ///
 /// This trait must be implemented for any types that are used as counter
@@ -437,11 +491,16 @@ where
 /// client.count("some.counter", 1).unwrap();
 /// client.time("some.timer", 42).unwrap();
 /// client.time("some.timer", Duration::from_millis(42)).unwrap();
+/// client.time("some.timer", vec![42]).unwrap();
+/// client.time("some.timer", vec![Duration::from_millis(42)]).unwrap();
 /// client.gauge("some.gauge", 8).unwrap();
 /// client.meter("some.meter", 13).unwrap();
 /// client.histogram("some.histogram", 4).unwrap();
 /// client.histogram("some.histogram", Duration::from_nanos(4)).unwrap();
+/// client.histogram("some.histogram", vec![4]).unwrap();
+/// client.histogram("some.histogram", vec![Duration::from_nanos(4)]).unwrap();
 /// client.distribution("some.distribution", 4).unwrap();
+/// client.distribution("some.distribution", vec![4]).unwrap();
 /// client.set("some.set", 5).unwrap();
 /// ```
 pub trait MetricClient:
@@ -449,14 +508,21 @@ pub trait MetricClient:
     + CountedExt
     + Timed<u64>
     + Timed<Duration>
+    + Timed<Vec<u64>>
+    + Timed<Vec<Duration>>
     + Gauged<u64>
     + Gauged<f64>
     + Metered<u64>
     + Histogrammed<u64>
     + Histogrammed<f64>
     + Histogrammed<Duration>
+    + Histogrammed<Vec<u64>>
+    + Histogrammed<Vec<f64>>
+    + Histogrammed<Vec<Duration>>
     + Distributed<u64>
     + Distributed<f64>
+    + Distributed<Vec<u64>>
+    + Distributed<Vec<f64>>
     + Setted<i64>
     + Compat
 {
@@ -574,6 +640,8 @@ pub trait MetricBackend: Sealed {
 ///
 /// let client = StatsdClient::builder("prefix", NopMetricSink)
 ///     .with_error_handler(my_error_handler)
+///     .with_tag("environment", "production")
+///     .with_tag_value("rust")
 ///     .build();
 ///
 /// client.count("something", 123);
@@ -585,6 +653,7 @@ pub struct StatsdClientBuilder {
     prefix: String,
     sink: Box<dyn MetricSink + Sync + Send + RefUnwindSafe>,
     errors: Box<dyn Fn(MetricError) + Sync + Send + RefUnwindSafe>,
+    tags: Vec<(Option<String>, String)>,
 }
 
 impl StatsdClientBuilder {
@@ -600,6 +669,7 @@ impl StatsdClientBuilder {
 
             // optional with defaults
             errors: Box::new(nop_error_handler),
+            tags: Vec::new(),
         }
     }
 
@@ -620,6 +690,27 @@ impl StatsdClientBuilder {
         self
     }
 
+    /// Add a default tag with key and value to every metric published by the
+    /// built [StatsdClient].
+    pub fn with_tag<K, V>(mut self, key: K, value: V) -> Self
+    where
+        K: ToString,
+        V: ToString,
+    {
+        self.tags.push((Some(key.to_string()), value.to_string()));
+        self
+    }
+
+    /// Add a default tag with only a value to every metric published by the built
+    /// [StatsdClient].
+    pub fn with_tag_value<K>(mut self, value: K) -> Self
+    where
+        K: ToString,
+    {
+        self.tags.push((None, value.to_string()));
+        self
+    }
+
     /// Construct a new `StatsdClient` instance based on current settings.
     pub fn build(self) -> StatsdClient {
         StatsdClient::from_builder(self)
@@ -720,6 +811,7 @@ pub struct StatsdClient {
     prefix: String,
     sink: Box<dyn MetricSink + Sync + Send + RefUnwindSafe>,
     errors: Box<dyn Fn(MetricError) + Sync + Send + RefUnwindSafe>,
+    tags: Vec<(Option<String>, String)>,
 }
 
 impl StatsdClient {
@@ -863,8 +955,13 @@ impl StatsdClient {
             prefix: builder.prefix,
             sink: builder.sink,
             errors: builder.errors,
+            tags: builder.tags,
         }
     }
+
+    fn tags(&self) -> impl IntoIterator<Item = (Option<&str>, &str)> {
+        self.tags.iter().map(|(k, v)| (k.as_deref(), v.as_str()))
+    }
 }
 
 impl Sealed for StatsdClient {}
@@ -888,8 +985,8 @@ impl fmt::Debug for StatsdClient {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
-            "StatsdClient {{ prefix: {:?}, sink: ..., errors: ... }}",
-            self.prefix
+            "StatsdClient {{ prefix: {:?}, sink: ..., errors: ..., tags: {:?} }}",
+            self.prefix, self.tags,
         )
     }
 }
@@ -900,7 +997,9 @@ where
 {
     fn count_with_tags<'a>(&'a self, key: &'a str, value: T) -> MetricBuilder<'_, '_, Counter> {
         match value.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::counter(&self.prefix, key, v), self),
+            Ok(v) => {
+                MetricBuilder::from_fmt(MetricFormatter::counter(&self.prefix, key, v), self).with_tags(self.tags())
+            }
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -914,7 +1013,7 @@ where
 {
     fn time_with_tags<'a>(&'a self, key: &'a str, time: T) -> MetricBuilder<'_, '_, Timer> {
         match time.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::timer(&self.prefix, key, v), self),
+            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::timer(&self.prefix, key, v), self).with_tags(self.tags()),
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -926,7 +1025,7 @@ where
 {
     fn gauge_with_tags<'a>(&'a self, key: &'a str, value: T) -> MetricBuilder<'_, '_, Gauge> {
         match value.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::gauge(&self.prefix, key, v), self),
+            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::gauge(&self.prefix, key, v), self).with_tags(self.tags()),
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -938,7 +1037,7 @@ where
 {
     fn meter_with_tags<'a>(&'a self, key: &'a str, value: T) -> MetricBuilder<'_, '_, Meter> {
         match value.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::meter(&self.prefix, key, v), self),
+            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::meter(&self.prefix, key, v), self).with_tags(self.tags()),
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -950,7 +1049,9 @@ where
 {
     fn histogram_with_tags<'a>(&'a self, key: &'a str, value: T) -> MetricBuilder<'_, '_, Histogram> {
         match value.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::histogram(&self.prefix, key, v), self),
+            Ok(v) => {
+                MetricBuilder::from_fmt(MetricFormatter::histogram(&self.prefix, key, v), self).with_tags(self.tags())
+            }
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -974,7 +1075,7 @@ where
 {
     fn set_with_tags<'a>(&'a self, key: &'a str, value: T) -> MetricBuilder<'_, '_, Set> {
         match value.try_to_value() {
-            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::set(&self.prefix, key, v), self),
+            Ok(v) => MetricBuilder::from_fmt(MetricFormatter::set(&self.prefix, key, v), self).with_tags(self.tags()),
             Err(e) => MetricBuilder::from_error(e, self),
         }
     }
@@ -994,6 +1095,7 @@ mod tests {
     };
     use crate::sinks::{MetricSink, NopMetricSink, QueuingMetricSink, SpyMetricSink};
     use crate::types::{ErrorKind, Metric, MetricError};
+    use crate::StatsdClientBuilder;
     use std::io;
     use std::panic::RefUnwindSafe;
     use std::sync::atomic::{AtomicUsize, Ordering};
@@ -1009,6 +1111,25 @@ mod tests {
         assert_eq!("some.method:1|c", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_merging_default_tags_with_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("hello", "world")
+            .with_tag_value("production")
+            .build();
+        let res = client
+            .count_with_tags("some.counter", 3)
+            .with_tag("foo", "bar")
+            .with_tag_value("fizz")
+            .with_tag("bucket", "123")
+            .try_send();
+
+        assert_eq!(
+            "prefix.some.counter:3|c|#hello:world,production,foo:bar,fizz,bucket:123",
+            res.unwrap().as_metric_str()
+        );
+    }
+
     #[test]
     fn test_statsd_client_count_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1020,6 +1141,16 @@ mod tests {
         assert_eq!("prefix.some.counter:3|c|#foo:bar", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_count_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("hello", "world")
+            .build();
+        let res = client.count_with_tags("some.counter", 3).try_send();
+
+        assert_eq!("prefix.some.counter:3|c|#hello:world", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_incr_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1028,6 +1159,16 @@ mod tests {
         assert_eq!("prefix.some.counter:1|c|#foo:bar", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_incr_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.incr_with_tags("some.counter").try_send();
+
+        assert_eq!("prefix.some.counter:1|c|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_decr_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1036,6 +1177,16 @@ mod tests {
         assert_eq!("prefix.some.counter:-1|c|#foo:bar", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_decr_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.decr_with_tags("some.counter").try_send();
+
+        assert_eq!("prefix.some.counter:-1|c|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_gauge_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1051,6 +1202,16 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_statsd_client_gauge_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.gauge_with_tags("some.gauge", 4).try_send();
+
+        assert_eq!("prefix.some.gauge:4|g|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_time_duration() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1059,6 +1220,19 @@ mod tests {
         assert_eq!("prefix.key:157|ms", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_time_multiple_durations() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_millis(157),
+            Duration::from_millis(158),
+            Duration::from_millis(159),
+        ];
+        let res = client.time("key", durations);
+
+        assert_eq!("prefix.key:157:158:159|ms", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_time_duration_with_overflow() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1067,6 +1241,19 @@ mod tests {
         assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind())
     }
 
+    #[test]
+    fn test_statsd_client_time_multiple_durations_with_overflow() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_millis(157),
+            Duration::from_secs(u64::MAX),
+            Duration::from_millis(159),
+        ];
+        let res = client.time("key", durations);
+
+        assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind())
+    }
+
     #[test]
     fn test_statsd_client_time_duration_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1079,6 +1266,33 @@ mod tests {
         assert_eq!("prefix.key:157|ms|#foo:bar,quux", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_time_duration_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.time("key", Duration::from_millis(157));
+
+        assert_eq!("prefix.key:157|ms|#foo:bar", res.unwrap().as_metric_str());
+    }
+
+    #[test]
+    fn test_statsd_client_time_multiple_durations_with_tags() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_millis(157),
+            Duration::from_millis(158),
+            Duration::from_millis(159),
+        ];
+        let res = client
+            .time_with_tags("key", durations)
+            .with_tag("foo", "bar")
+            .with_tag_value("quux")
+            .try_send();
+
+        assert_eq!("prefix.key:157:158:159|ms|#foo:bar,quux", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_time_duration_with_tags_with_overflow() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1092,6 +1306,24 @@ mod tests {
         assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind());
     }
 
+    #[test]
+    fn test_statsd_client_time_multiple_durations_with_tags_with_overflow() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_millis(157),
+            Duration::from_secs(u64::MAX),
+            Duration::from_millis(159),
+        ];
+        let res = client
+            .time_with_tags("key", durations)
+            .with_tag("foo", "bar")
+            .with_tag_value("quux")
+            .try_send();
+
+        assert!(res.is_err());
+        assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind());
+    }
+
     #[test]
     fn test_statsd_client_meter_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1104,6 +1336,16 @@ mod tests {
         assert_eq!("prefix.some.meter:64|m|#segment:142,beta", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_meter_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.meter_with_tags("some.meter", 64).try_send();
+
+        assert_eq!("prefix.some.meter:64|m|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_histogram_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1119,6 +1361,24 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_statsd_client_histogram_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.histogram_with_tags("some.histo", 27).try_send();
+
+        assert_eq!("prefix.some.histo:27|h|#foo:bar", res.unwrap().as_metric_str());
+    }
+
+    #[test]
+    fn test_statsd_client_histogram_with_multiple_values() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let res = client.histogram_with_tags("some.histo", vec![27, 28, 29]).try_send();
+
+        assert_eq!("prefix.some.histo:27:28:29|h", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_histogram_duration() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1127,6 +1387,19 @@ mod tests {
         assert_eq!("prefix.key:210|h", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_histogram_multiple_durations() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_nanos(210),
+            Duration::from_nanos(211),
+            Duration::from_nanos(212),
+        ];
+        let res = client.histogram("key", durations);
+
+        assert_eq!("prefix.key:210:211:212|h", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_histogram_duration_with_overflow() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1135,6 +1408,20 @@ mod tests {
         assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind());
     }
 
+    #[test]
+    fn test_statsd_client_histogram_multiple_durations_with_overflow() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let durations = vec![
+            Duration::from_nanos(210),
+            Duration::from_secs(u64::MAX),
+            Duration::from_nanos(212),
+        ];
+
+        let res = client.histogram("key", durations);
+
+        assert_eq!(ErrorKind::InvalidInput, res.unwrap_err().kind());
+    }
+
     #[test]
     fn test_statsd_client_histogram_duration_with_tags() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1147,6 +1434,16 @@ mod tests {
         assert_eq!("prefix.key:4096|h|#foo:bar,beta", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_histogram_duration_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.histogram_with_tags("key", Duration::from_nanos(4096)).try_send();
+
+        assert_eq!("prefix.key:4096|h|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_histogram_duration_with_tags_with_overflow() {
         let client = StatsdClient::from_sink("prefix", NopMetricSink);
@@ -1174,6 +1471,21 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_statsd_client_distribution_multiple_values_with_tags() {
+        let client = StatsdClient::from_sink("prefix", NopMetricSink);
+        let res = client
+            .distribution_with_tags("some.distr", vec![27, 28, 29])
+            .with_tag("host", "www03.example.com")
+            .with_tag_value("rc1")
+            .try_send();
+
+        assert_eq!(
+            "prefix.some.distr:27:28:29|d|#host:www03.example.com,rc1",
+            res.unwrap().as_metric_str()
+        );
+    }
+
     #[test]
     fn test_statsd_client_set_with_tags() {
         let client = StatsdClient::from_sink("myapp", NopMetricSink);
@@ -1182,6 +1494,16 @@ mod tests {
         assert_eq!("myapp.some.set:3|s|#foo:bar", res.unwrap().as_metric_str());
     }
 
+    #[test]
+    fn test_statsd_client_set_with_default_tags() {
+        let client = StatsdClientBuilder::new("prefix", NopMetricSink)
+            .with_tag("foo", "bar")
+            .build();
+        let res = client.set_with_tags("some.set", 3).try_send();
+
+        assert_eq!("prefix.some.set:3|s|#foo:bar", res.unwrap().as_metric_str());
+    }
+
     #[test]
     fn test_statsd_client_with_tags_send_success() {
         let (rx, sink) = SpyMetricSink::new();
@@ -1251,6 +1573,14 @@ mod tests {
         client.time("some.timer", Duration::from_millis(20)).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_timed_packed_duration() {
+        let client: Box<dyn Timed<Vec<Duration>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+        let durations = vec![Duration::from_millis(20), Duration::from_millis(21)];
+
+        client.time("some.timer", durations).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_gauged_u64() {
         let client: Box<dyn Gauged<u64>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
@@ -1279,6 +1609,13 @@ mod tests {
         client.histogram("some.histogram", 4).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_histogrammed_packed_u64() {
+        let client: Box<dyn Histogrammed<Vec<u64>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+
+        client.histogram("some.histogram", vec![4, 5, 6]).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_histogrammed_f64() {
         let client: Box<dyn Histogrammed<f64>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
@@ -1286,6 +1623,13 @@ mod tests {
         client.histogram("some.histogram", 4.0).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_histogrammed_packed_f64() {
+        let client: Box<dyn Histogrammed<Vec<f64>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+
+        client.histogram("some.histogram", vec![4.0, 5.0, 6.0]).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_histogrammed_duration() {
         let client: Box<dyn Histogrammed<Duration>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
@@ -1293,6 +1637,14 @@ mod tests {
         client.histogram("some.histogram", Duration::from_nanos(4)).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_histogrammed_packed_duration() {
+        let client: Box<dyn Histogrammed<Vec<Duration>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+        let durations = vec![Duration::from_nanos(4), Duration::from_nanos(5)];
+
+        client.histogram("some.histogram", durations).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_distributed_u64() {
         let client: Box<dyn Distributed<u64>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
@@ -1300,6 +1652,13 @@ mod tests {
         client.distribution("some.distribution", 33).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_distributed_packed_u64() {
+        let client: Box<dyn Distributed<Vec<u64>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+
+        client.distribution("some.distribution", vec![33, 34]).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_distributed_f64() {
         let client: Box<dyn Distributed<f64>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
@@ -1307,6 +1666,13 @@ mod tests {
         client.distribution("some.distribution", 33.0).unwrap();
     }
 
+    #[test]
+    fn test_statsd_client_as_distributed_packed_f64() {
+        let client: Box<dyn Distributed<Vec<f64>>> = Box::new(StatsdClient::from_sink("prefix", NopMetricSink));
+
+        client.distribution("some.distribution", vec![33.0, 34.0]).unwrap();
+    }
+
     #[test]
     fn test_statsd_client_as_setted() {
         let client: Box<dyn Setted<i64>> = Box::new(StatsdClient::from_sink("myapp", NopMetricSink));
@@ -1324,14 +1690,23 @@ mod tests {
         client.count("some.counter", 3).unwrap();
         client.time("some.timer", 198).unwrap();
         client.time("some.timer", Duration::from_millis(198)).unwrap();
+        client.time("some.timer", vec![198]).unwrap();
+        client.time("some.timer", vec![Duration::from_millis(198)]).unwrap();
         client.gauge("some.gauge", 4).unwrap();
         client.gauge("some.gauge", 4.0).unwrap();
         client.meter("some.meter", 29).unwrap();
         client.histogram("some.histogram", 32).unwrap();
         client.histogram("some.histogram", 32.0).unwrap();
         client.histogram("some.histogram", Duration::from_nanos(32)).unwrap();
+        client.histogram("some.histogram", vec![32]).unwrap();
+        client.histogram("some.histogram", vec![32.0]).unwrap();
+        client
+            .histogram("some.histogram", vec![Duration::from_nanos(32)])
+            .unwrap();
         client.distribution("some.distribution", 248).unwrap();
         client.distribution("some.distribution", 248.0).unwrap();
+        client.distribution("some.distribution", vec![248]).unwrap();
+        client.distribution("some.distribution", vec![248.0]).unwrap();
         client.set("some.set", 5).unwrap();
     }
 }
